/*
   GNUstep ProjectCenter - http://www.gnustep.org/experience/ProjectCenter.html

   Copyright (C) 2000-2007 Free Software Foundation

   Authors: Philippe C.D. Robert
            Sergii Stoian

   This file is part of GNUstep.

   This application is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This application is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU General Public
   License along with this library; if not, write to the Free
   Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA.
*/

#import <AppKit/AppKit.h>

@class PCProject;

@interface PCProjectBuilderOptions : NSObject
{
  PCProject     *project;
  id            delegate;           // Usually PCProjectBuilder

  NSPanel       *optionsPanel;
  NSPopUpButton *targetPopup;
  NSTextField   *buildArgsField;
  NSButton      *verboseButton;     // messages=yes
  NSButton      *debugButton;       // debug=no
  NSButton      *stripButton;       // strip=yes
  NSButton      *sharedLibsButton;  // shared=no
}

- (id)initWithProject:(PCProject *)aProject delegate:(id)aDelegate;
- (void)show:(NSRect)builderRect;
- (NSString *)buildTarget;

- (void)loadProjectProperties:(NSNotification *)aNotif;

@end

@interface PCProjectBuilderOptions (Delegate)

- (void)targetDidSet:(NSString *)aTarget;

@end

